use crate::schema;

pub fn list(realm: &str, db: microrm::DB) {
    // get realm ID
    let qi = db.query_interface();

    let realm_id = qi.get_one_by(schema::RealmColumns::Shortname, realm).expect("No such realm").id();

    let users = qi.get_all_by(schema::UserColumns::Realm, realm_id).expect("Can't get list of users");

    println!("User list ({} users):", users.len());

    for user in &users {
        println!("- {:20}", user.username);
        let auth_challenges = qi.get_all_by(schema::AuthChallengeColumns::User, user.id()).expect("Can't get authentication challenges?");
        for ch in &auth_challenges {
            println!("    - Has {:?} authentication challenge", ch.challenge_type);
        }
    }
}

pub fn create(realm: &str, db: microrm::DB, username: &str) {
    // get realm ID
    let qi = db.query_interface();

    let realm_id = qi.get_one_by(schema::RealmColumns::Shortname, realm).expect("No such realm").id();

    // check that the user doesn't exist already
    let existing_user = qi.get_one_by_multi(&[schema::UserColumns::Realm, schema::UserColumns::Username], &microrm::value_list![&realm_id, &username]);

    if existing_user.is_some() {
        log::error!("Can't create user {} in {} realm as a user with that username already exists", username, realm);
        return;
    }

    qi.add(&schema::User { realm: realm_id, username: username.to_owned() });
}

pub fn change_auth(realm: &str, db: microrm::DB, username: &str, change_password: bool) {
    // get realm ID
    let qi = db.query_interface();

    let realm_id = qi.get_one_by(schema::RealmColumns::Shortname, realm).expect("No such realm").id();
    // check that the user exists
    let existing_user = qi.get_one_by_multi(&[schema::UserColumns::Realm, schema::UserColumns::Username], &microrm::value_list![&realm_id, &username]);
    if existing_user.is_none() {
        log::error!("User {} does not exist in the {} realm!", username, realm);
        return;
    }

    let user = existing_user.unwrap();

    if change_password {
        qi.delete_by_multi(&[schema::AuthChallengeColumns::User, schema::AuthChallengeColumns::ChallengeType], &microrm::value_list![&user.id(), &schema::AuthChallengeType::Password]).expect("Couldn't delete existing password?");

        let raw_pass = rpassword::prompt_password("Enter new user password: ").unwrap();

        // store hashed password
        use ring::pbkdf2;

        let rng = ring::rand::SystemRandom::new();
        let salt: [u8; 16] = ring::rand::generate(&rng).expect("Couldn't generate random salt?").expose();

        let mut generated = [0u8; ring::digest::SHA256_OUTPUT_LEN];

        pbkdf2::derive(pbkdf2::PBKDF2_HMAC_SHA256, std::num::NonZeroU32::new(20000).unwrap(), &salt, raw_pass.as_bytes(), &mut generated);

        qi.add(&schema::AuthChallenge { user: user.id(), challenge_type: schema::AuthChallengeType::Password, public: salt.into(), secret: generated.into() });
    }
}
